package sequence;

import java.rmi.RemoteException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.naming.NamingException;
import javax.sql.DataSource;

import util.EJBHomeFactory;
import util.HomeFactoryException;

/**
 * Sequence Block Bean
 * 
 * @author ٺΰ
 * @since 2005. 3. 30.
 */
public class SequenceBean implements EntityBean {
    public String name;

    public int number;

    private Connection con = null;

    private EntityContext ctx;

    /**
     * ο Entity Bean ̺ Ѵ.
     * 
     * @param name
     * @param number
     * @return
     * @throws CreateException
     */
    public String ejbCreate(String name, Integer number) throws CreateException {
        setName(name);
        setNumber(number.intValue());

        try {
            String query = "INSERT INTO Sequence (name, number) VALUES (?, ?)";
            PreparedStatement pstmt = con.prepareStatement(query);
            pstmt.setString(1, name);
            pstmt.setInt(2, number.intValue());
            pstmt.executeUpdate();
            pstmt.close();
        } catch (Exception e) {
            System.err.println("ejbCreate() : " + e);
        }

        return name;
    }

    public void ejbPostCreate(String name, Integer number)
            throws CreateException {
    }

    public String ejbFindByPrimaryKey(String name) throws FinderException {
        String result = null;
        try {
            String query = "SELECT name FROM Sequence WHERE name = ?";
            PreparedStatement pstmt = con.prepareStatement(query);
            pstmt.setString(1, name);
            if (pstmt.executeQuery() != null) {
                result = name;
            }
            pstmt.close();
        } catch (Exception e) {
            System.err.println("ejbFindByPrimaryKey() : " + e);
        }
        return result;
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setNumber(int number) {
        this.number = number;
    }

    public String getName() {
        return name;
    }

    public int getNumber() {
        return number;
    }

    public int getIncrementData(int block) {
        setNumber(getNumber() + block);

        return getNumber();
    }

    /**
     * DB Connection...
     * 
     * @see javax.ejb.EntityBean#setEntityContext(javax.ejb.EntityContext)
     * @param ctx
     * @throws EJBException
     * @throws RemoteException
     */
    public void setEntityContext(EntityContext ctx) throws EJBException,
            RemoteException {
        this.ctx = ctx;

        try {
            EJBHomeFactory homeFactory = EJBHomeFactory.getFactory();
            DataSource dataSource = (DataSource) homeFactory
                    .lookUpDataSource("jdbc/BonebaDB");
            con = dataSource.getConnection();
        } catch (HomeFactoryException e) {
            System.out.println("HomeFactoryException : \n" + e.getMessage());
        } catch (NamingException e) {
            System.out.println("NamingException : \n" + e.getMessage());
        } catch (SQLException e) {
            System.out.println("SQLException : \n" + e.getMessage());
        }
    }

    /**
     * DB disconnection..
     * 
     * @see javax.ejb.EntityBean#unsetEntityContext()
     * @throws EJBException
     * @throws RemoteException
     */
    public void unsetEntityContext() throws EJBException, RemoteException {
        this.ctx = null;

        try {
            if (con != null) {
                con.close();
            }
        } catch (SQLException e1) {
            System.out.println("SQLException : \n" + e1.getMessage());
        }
    }

    /**
     * Bean  ش ̺ 
     * 
     * @see javax.ejb.EntityBean#ejbRemove()
     * @throws RemoveException
     * @throws EJBException
     * @throws RemoteException
     */
    public void ejbRemove() throws RemoveException, EJBException,
            RemoteException {
        try {
            String query = "DELETE FROM Sequence WHERE name = ?";
            PreparedStatement pstmt = con.prepareStatement(query);
            pstmt.setString(1, name);
            pstmt.executeUpdate();
            pstmt.close();
        } catch (Exception e) {
            System.err.println("ejbRemove() : " + e);
        }
    }

    /**
     * Bean νϽ Ȱȭ Ų.
     * 
     * @see javax.ejb.EntityBean#ejbActivate()
     * @throws EJBException
     * @throws RemoteException
     */
    public void ejbActivate() throws EJBException, RemoteException {
        name = (String) ctx.getPrimaryKey();
    }

    /**
     * Bean νϽ Ȱȭ Ų.
     * 
     * @see javax.ejb.EntityBean#ejbPassivate()
     * @throws EJBException
     * @throws RemoteException
     */
    public void ejbPassivate() throws EJBException, RemoteException {
        name = null;
    }

    /**
     * Bean νϽ  ´.
     * 
     * @see javax.ejb.EntityBean#ejbLoad()
     * @throws EJBException
     * @throws RemoteException
     */
    public void ejbLoad() throws EJBException, RemoteException {
        try {
            String query = "SELECT name, number FROM Sequence WHERE name = ?";
            PreparedStatement pstmt = con.prepareStatement(query);
            pstmt.setString(1, name);
            ResultSet rs = pstmt.executeQuery();

            if (rs.next()) {
                name = rs.getString("name");
                number = rs.getInt("number");
            }
            rs.close();
            pstmt.close();
        } catch (Exception e) {
            System.err.println("ejbLoad() : " + e);
        }
    }

    /**
     * Bean νϽ  ش ̺ Ѵ.
     * 
     * @see javax.ejb.EntityBean#ejbStore()
     * @throws EJBException
     * @throws RemoteException
     */
    public void ejbStore() throws EJBException, RemoteException {
        try {
            String query = "UPDATE Sequence SET number = ? WHERE name = ?";
            PreparedStatement pstmt = con.prepareStatement(query);
            pstmt.setInt(1, number);
            pstmt.setString(2, name);
            pstmt.executeUpdate();
            pstmt.close();
        } catch (Exception e) {
            System.err.println("SequenceBean.ejbStore() : " + e);
        }
    }
}