package util;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.ejb.EJBHome;
import javax.ejb.EJBLocalHome;
import javax.mail.Session;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import javax.sql.DataSource;

/**
 * EJB Home Factory, EJBHome ܼ ؽø ĳø Ѵ. NamingException  ܵ Ŭ̾Ʈ
 *  ܼȭϱ  丮 ܷ   ִ.
 * 
 * @author ٺΰ
 * @since 2005. 3. 30.
 */
public class EJBHomeFactory {
    private Map ejbHomes;

    private Context initialCtx;

    private static EJBHomeFactory instance;

    /**
     * 
     * 
     * @throws NamingException
     */
    private EJBHomeFactory() throws NamingException {
        initialCtx = new InitialContext();
        ejbHomes = Collections.synchronizedMap(new HashMap());
    }

    public static EJBHomeFactory getFactory() throws HomeFactoryException {
        if (instance == null) {
            try {
                instance = new EJBHomeFactory();
            } catch (NamingException e) {
                throw new HomeFactoryException(e);
            }
        }

        return instance;
    }

    /**
     * DataSource ãƿ´. ̰͵ 淡 .
     * 
     * @param jndiName
     * @return javax.sql.DataSource
     * @throws NamingException
     */
    public DataSource lookUpDataSource(String jndiName) throws NamingException {
        // ĳ ȿ DataSource ã´.
        DataSource dataSource = (DataSource) ejbHomes.get(DataSource.class);

        // ĳÿ  ĳÿ ϰ ߰Ѵ.
        if (dataSource == null) {
            dataSource = (DataSource) initialCtx.lookup(jndiName);
            ejbHomes.put(DataSource.class, dataSource);
        }

        // DataSource .
        return dataSource;
    }

    /**
     * Session ãƿ´. 淡 ɱϰ  ô.
     * 
     * @param jndiName
     * @return javax.mail.Session
     * @throws NamingException
     */
    public Session lookUpSession(String jndiName) throws NamingException {
        // ĳ ȿ Session ã´.
        Session session = (Session) ejbHomes.get(Session.class);

        // ĳÿ  ĳÿ ϰ ߰Ѵ.
        if (session == null) {
            session = (Session) initialCtx.lookup(jndiName);
            ejbHomes.put(Session.class, session);
        }

        // Session .
        return session;
    }

    /**
     * Home Interface ãƿ´.
     * 
     * @param jndiName
     * @param homeClass
     * @return EJBHome
     * @throws NamingException
     */
    public EJBHome lookUpHome(String jndiName, Class homeClass)
            throws NamingException {
        if (homeClass == null) {
            throw new IllegalArgumentException(
                    "supplied home class to lookup was null");
        }

        // ĳ ȿ EJB Ȩ ã´.
        EJBHome ejbHome = (EJBHome) ejbHomes.get(homeClass);

        // ĳÿ  ĳÿ ϰ ߰Ѵ.
        if (ejbHome == null) {
            ejbHome = (EJBHome) PortableRemoteObject.narrow(initialCtx
                    .lookup(jndiName), homeClass);
            ejbHomes.put(homeClass, ejbHome);
        }

        // Ư Ȩ ̽ .
        return ejbHome;
    }

    /**
     * Local Home Interface ãƿ´.
     * 
     * @param jndiName
     * @param homeClass
     * @return EJBLocalHome
     * @throws NamingException
     */
    public EJBLocalHome lookUpLocalHome(String jndiName, Class homeClass)
            throws NamingException {
        if (homeClass == null) {
            throw new IllegalArgumentException(
                    "supplied home class to lookup was null");
        }

        // ĳ ȿ EJB Ȩ ã´.
        EJBLocalHome ejbLocalHome = (EJBLocalHome) ejbHomes.get(homeClass);

        // ĳÿ  ĳÿ ϰ ߰Ѵ.
        if (ejbLocalHome == null) {
            ejbLocalHome = (EJBLocalHome) PortableRemoteObject.narrow(
                    initialCtx.lookup(jndiName), homeClass);
            ejbHomes.put(homeClass, ejbLocalHome);
        }

        // Ư Ȩ ̽ .
        return ejbLocalHome;
    }
}