/*
 * Created on 2005. 4. 12. 
 */
package viewer.jframe;

import java.io.File;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import popFolder.FileList;
import popFolder.PopFolder;
import popFolder.PopFolderHome;

/**
 * Viewer Control..
 * 
 * @author BABU
 * @since 2005. 4. 12.
 */
public class ViewerControl implements Runnable {
    private Thread thread = null;

    private String uid;

    private PopFolderViewer viewer;

    public String rootPath;

    private PopFolder pf;

    private int type = 0;

    public final static int CONNECT = 1;

    public final static int DISCONNECT = 2;

    public final static int GET_FILE_LIST = 3;

    public final static int GET_FILE = 4;

    public final static int MKDIR = 5;

    public final static int DELETE = 6;

    public final static int RENAME = 7;

    /**
     * 
     * 
     * @param viewer
     */
    public ViewerControl(PopFolderViewer viewer, String uid) {
        this.viewer = viewer;
        this.uid = uid;
    }

    /**
     *  
     * 
     * @param type
     */
    public void start(int type) {
        this.type = type;

        thread = new Thread(this);
        viewer.getAlertDialog();
        thread.start();
    }

    /**
     * EJB ϰ rootPath ´.
     * 
     * @return
     */
    public boolean connect() {
        viewer.setAlert(4, 0, "EJB ..");
        boolean flag = false;

        try {
            Properties prop = new Properties();
            prop.put("javax.rmi.CORBA.UtilClass",
                    "com.sun.corba.ee.impl.javax.rmi.CORBA.Util");
            prop.put("org.omg.CORBA.ORBClass",
                    "com.sun.corba.ee.impl.orb.ORBImpl");
            prop.put("org.omg.CORBA.ORBSingletonClass",
                    "com.sun.corba.ee.impl.orb.ORBSingleton");
            prop.put("java.naming.factory.initial",
                    "com.sun.jndi.cosnaming.CNCtxFactory");
            //prop.put("org.omg.CORBA.ORBInitialHost", "218.39.199.58");
            prop.put("org.omg.CORBA.ORBInitialHost", "localhost");
            prop.put("org.omg.CORBA.ORBInitialPort", "3700");

            InitialContext ctx = new InitialContext(prop);
            viewer.setAlert(4, 1, "EJB ..");

            PopFolderHome pfh = (PopFolderHome) PortableRemoteObject.narrow(ctx
                    .lookup("PopFolderBean"), PopFolderHome.class);
            viewer.setAlert(4, 3, "EJB ..");

            pf = pfh.create();

            viewer.setAlert(4, 4, "EJB ..");

            // ROOT PATH 
            rootPath = pf.getRootPath(uid);
        } catch (Exception e) {
            System.err.println("connect() : " + e);
        }

        return flag;
    }

    public void getFileList(TreePath path) {
        try {
            String filePath = rootPath;
            StringTokenizer st = new StringTokenizer(path.toString(), "[,]");

            if (st.countTokens() > 1) {
                st.nextToken();

                while (st.hasMoreTokens()) {
                    filePath = filePath + File.separator
                            + st.nextToken().trim();
                }
            }

            DefaultMutableTreeNode selnode = (DefaultMutableTreeNode) path
                    .getLastPathComponent();
            selnode.removeAllChildren();

            // ϴ Ʈ 
            Vector vector = pf.getFileList(filePath);

            viewer.model.removeAllRow();

            for (int i = 0; i < vector.size(); i++) {
                FileList fileList = (FileList) vector.elementAt(i);

                if (fileList.isDirectory()) {
                    DefaultMutableTreeNode node = new DefaultMutableTreeNode(
                            fileList.getName());
                    selnode.add(node);
                } else {
                    Object[] obj = { fileList.getName(),
                            new Long(fileList.getLength()) };
                    viewer.model.addRow(obj);
                }
            }

            //viewer.jTree.updateUI();
            //viewer.jTable.updateUI();
        } catch (Exception e) {
            System.out.println("ViewerControl getFileList() : " + e);
        }
    }

    /**
     *  
     * 
     * @return
     */
    public boolean disconnect() {
        try {
            rootPath = null;
            uid = null;
            pf.remove();
        } catch (Exception e) {
            return false;
        }
        return true;
    }

    /**
     * @see java.lang.Runnable#run()
     *  
     */
    public void run() {
        switch (type) {
        case CONNECT:
            connect();
            break;
        }

        viewer.closeAlert();
    }

    /**
     *  
     * 
     * @param path
     */
    public void deleteFolder(TreePath path) {
        try {
            String filePath = rootPath;
            StringTokenizer st = new StringTokenizer(path.toString(), "[,]");

            if (st.countTokens() > 1) {
                st.nextToken();

                while (st.hasMoreTokens()) {
                    filePath = filePath + File.separator
                            + st.nextToken().trim();
                }
            }

            pf.delete(filePath);

            getFileList(path.getParentPath());
            viewer.jTree.updateUI();
        } catch (Exception e) {
            System.out.println("ViewerControl deleteFolder() : " + e);
        }
    }

    /**
     * @param path
     * @param rename
     */
    public void renameFolder(TreePath path, String rename) {
        try {
            String filePath = rootPath;
            String rePath = rootPath;
            StringTokenizer st = new StringTokenizer(path.toString(), "[,]");

            int count = st.countTokens();

            if (count > 1) {
                st.nextToken();

                for (int i = 1; i < count; i++) {
                    filePath = filePath + File.separator
                            + st.nextToken().trim();

                    if (i == (count - 1)) {
                        rePath = rePath + File.separator + rename;
                    } else {
                        rePath = rePath + File.separator
                                + st.nextToken().trim();
                    }
                }
            }

            pf.rename(filePath, rePath);

            getFileList(path.getParentPath());
            viewer.jTree.updateUI();
        } catch (Exception e) {
            System.out.println("ViewerControl renameFolder() : " + e);
        }
    }

    /**
     * @param selectionPath
     * @param fileName
     */
    public void deleteFile(TreePath path, String fileName) {
        try {
            String filePath = rootPath;
            StringTokenizer st = new StringTokenizer(path.toString(), "[,]");

            if (st.countTokens() > 1) {
                st.nextToken();

                while (st.hasMoreTokens()) {
                    filePath = filePath + File.separator
                            + st.nextToken().trim();
                }
            }

            pf.delete(filePath + File.separator + fileName);

            getFileList(path);
        } catch (Exception e) {
            System.out.println("ViewerControl deleteFile() : " + e);
        }
    }

    /**
     * @param path
     * @param fileName
     * @param rename
     */
    public void renameFile(TreePath path, String fileName, String rename) {
        try {
            String filePath = rootPath;
            StringTokenizer st = new StringTokenizer(path.toString(), "[,]");

            if (st.countTokens() > 1) {
                st.nextToken();

                while (st.hasMoreTokens()) {
                    filePath = filePath + File.separator
                            + st.nextToken().trim();
                }
            }

            pf.rename(filePath + File.separator + fileName, filePath
                    + File.separator + rename);

            getFileList(path);
        } catch (Exception e) {
            System.out.println("ViewerControl deleteFile() : " + e);
        }
    }
}